@@ -6,6 +6,7 @@ from read_vme_offline.version import __version__
 #print("i... module read_vme_offline/ascdf is being run")
 
 import pandas as pd
+#import modin.pandas as pd
 import tables # here, to avoid a crash when writing pandas
 import h5py
 import datetime
@@ -206,19 +207,30 @@ def column_to_histo( dfcol , binmax = 32*1024, himax=32*1024, savename = "", hna
 
 #------------------------------------------------------------------------
 
-def save_to_tree(df2, filename, modname=""):
-    print("i... ===============creating and writing tree, expect 25% of the original size")
+def save_to_tree(df2, filename, modname="", treename="df"):
+    """
+     expect 25% of the original size
+    """
+    outfile =  os.path.splitext(filename)[0] +f"_tree{modname}.root"
+
+    print(f"i... ===============creating and writing {treename} tree", outfile)
     # convert to dict with numpy arrays
     #data = {key: df[key].values for key in ['time', 'E', 'x']}
     data = {key: df2[key].values for key in df2.columns }
     print("D... columns that go to TTree:", df2.columns)
     rdf = ROOT.RDF.MakeNumpyDataFrame( data )
     print(rdf)
-    outfile =  os.path.splitext(filename)[0] +f"_tree{modname}.root"
+
     print("D...  snaphost rdf as TTREE to ", outfile)
 
     start = dt.datetime.now()
-    rdf.Snapshot("df",outfile)
+
+    #ROOT.gInterpreter.Declare('ROOT::RDF::RSnapshotOptions opts;')
+    #opts.fMode = "update"
+    #print("D... opts=,", opts )
+    opts = ROOT.RDF.RSnapshotOptions()
+    opts.fMode =  "update"
+    rdf.Snapshot( treename ,  outfile, "", opts )
     meastime = (dt.datetime.now() - start).total_seconds()
     print(f"D... speed = {get_number_of_lines(filename)/meastime/1000/1000:.3f} Mrows/sec")
 
@@ -277,8 +289,13 @@ def enhance_by_dtus_and_next_E(df):
     if 'extras' in df.columns:
         print(f"D... broadening table - prev_dtus SATU")
         df['prev_satu'] = (df.time - df.time.shift(+1))*1000*1000
+        #  this says - if no saturation flag=> set 0
         df['prev_satu'] = np.where((df['extras']&1)==1, df['prev_satu'] , 0)
-        df.fillna(0)
+        #  same as
+        # df.loc[  (df.extras&1)!=1, 'prev_satu' ] =  0
+        #
+        #
+        df.fillna(0, inplace = True)
         df['prev_satu'] = df['prev_satu'].astype('float32') #
 
     print(df)
@@ -338,7 +355,7 @@ def pd_detect_zeroes(df, channel,  TIMEWIN_US = 25  ):
 
 #------------------------------------------------------------------------
 
-def pd_read_table(filename, sort = True):
+def pd_read_table(filename, sort = True,  fourcolumns = False):
     """
     READ THE ASC TABLE:   two possibilities:  4 columns OR 5 columns
 EARLIER:
@@ -363,6 +380,9 @@ register 0x1n80 to enable the fake-event saving in case of internal roll-over. I
 will have both bit[3] and bit[2] set to 1, while in the second case the event will have both bit[3] and
 bit[1] set to 1.
     """
+    print(f"D... fourcolumns=={fourcolumns}")
+    print(f"D... fourcolumns=={fourcolumns}")
+    print(f"D... fourcolumns=={fourcolumns}")
     start = dt.datetime.now()
     df = pd.read_table( filename,
                         names=['time','E','pu','ch','extras'],
@@ -372,6 +392,7 @@ bit[1] set to 1.
     meastime = (dt.datetime.now() - start).total_seconds()
     print(f"D... speed = {get_number_of_lines(filename)/meastime/1000/1000:.3f} Mrows/sec")
 
+    print(f"D... fourcolumns=={fourcolumns}")
 
     print("D... lasttime ",df.iloc[-1]["time"])
     print(freemem() )
@@ -391,6 +412,13 @@ bit[1] set to 1.
         df['extras'] = df['extras'].astype('int32')
     print("D...    finale types ************************************")
     print(df.dtypes)
+
+    if fourcolumns:
+       print("D... extra operation for four columns ... ")
+       print(df)
+       df['E'] = df['E']+ (df['pu']&1) * 16384
+       df['pu'] = df['pu'].apply(lambda x: x >> 1)
+       print(df)
     # print(freemem())
 
     #    df.to_hdf('run'+str(number)+'.h5',
