@@ -187,7 +187,7 @@ def fastread(filename, x=0, y=1, batch = 0, read_last=0, df = None, plot = True)
 
 #=====================================================================
 
-def coincidences(filename, ch0, ch1,  tree = False):
+def coincidences(filename, ch0, ch1,  tree = False, TIMESHIFT = 2):
     """
     """
     print( general.freemem() )
@@ -200,7 +200,7 @@ def coincidences(filename, ch0, ch1,  tree = False):
 
 
     #*************
-    df = general.pd_read_table(filename, sort = True)
+    df = general.pd_read_table(filename, sort = True,  fourcolumns = (ncolumns==4) )
     #print(df.dtypes)
     #print(df)
     chan_available = df['ch'].unique()
@@ -230,7 +230,7 @@ def coincidences(filename, ch0, ch1,  tree = False):
     #df1['time'] = np.where( df1['ch']==ch1,  df1['time'] ,  df1['time']+5.0)
 
 
-    TIMESHIFT = 2
+    # TIMESHIFT = 2
 
     df1.loc[ df1['ch'] ==ch1, 'time'] += TIMESHIFT*1e-6 # ADD  x us to ch1 time
     df1 = df1.sort_values(by="time")
@@ -269,10 +269,12 @@ def coincidences(filename, ch0, ch1,  tree = False):
 
     if tree:
         df1.rename(columns = {'E':f'E{ch0}','next_E':f'E{ch1}'}, inplace = True)
-        general.save_to_tree( df1, filename, modname=f"coin_{ch0}_{ch1}" )
+        general.save_to_tree( df1, filename, modname=f"coin" , treename = f"df{ch0}{ch1}")
 
     return
 
+
+
 #=============================== MAIN1 EVA ==================
 def eva_cut_time(filename,  chan, od=0, do=9999999 ,   tree = False):
     """
@@ -312,8 +314,8 @@ def eva_cut_time(filename,  chan, od=0, do=9999999 ,   tree = False):
 
 
     #*************
-    df = general.pd_read_table(filename, sort = True)
-    #print(df.dtypes)
+    df = general.pd_read_table(filename, sort = True, fourcolumns = (ncolumns==4) )
+    # df = general.pd_read_table(filename, sort = True, fourcolumns = False ) # we need to test PU and overflow with 4 colkumns only
     print(df)
 
     print("+"*50,"TOTALS")
@@ -362,10 +364,8 @@ def eva_cut_time(filename,  chan, od=0, do=9999999 ,   tree = False):
     chan_available = df1['ch'].unique()
     print("D... channels available:", chan_available, "   chan selected:", chan)
     print(f"D... pu        present: {df1['pu'].unique()}... 1 or 0 (3v0 before 2020)" )
-    if ncolumns == 5:
-        pileups_ch = len(df1.loc[ df1['pu']==1])
-    else:
-        pileups_ch = len(df1.loc[ df1['pu']==3])
+
+    pileups_ch = len(df1.loc[ df1['pu']==1])  # (3)for 4 columns, it is fixed in table_read
 
     print(f"D... pu        present: {df1['pu'].unique()}... 1 or 0 (3v0 before 2020)" )
     if 'extras' in df:
@@ -378,18 +378,26 @@ def eva_cut_time(filename,  chan, od=0, do=9999999 ,   tree = False):
 
     #*************  dt us    and next_E
     df1 = general.enhance_by_dtus_and_next_E(df1)
-    satu_n_ch = len(df1.loc[ (df1['extras']&1)!=0] )
-    print("D... SATURATIONS\n",df1.loc[ (df1['extras']&1)!=0] )
-    satu_DT_ch = df1.loc[ (df1['extras']&1)!=0]["prev_satu"].sum()
-    print(f"D... SATURATIONS {satu_n_ch} total: {satu_DT_ch/1e+6} sec.")
+    if 'extras' in  df1:
+        satu_n_ch = len(df1.loc[ (df1['extras']&1)!=0] )
+        print("D... SATURATIONS\n",df1.loc[ (df1['extras']&1)!=0] )
+        satu_DT_ch = df1.loc[ (df1['extras']&1)!=0]["prev_satu"].sum()/1e+6
+        print(f"D... SATURATIONS {satu_n_ch} total: {satu_DT_ch} sec.")
+    else:
+        satu_n_ch = None
+        satu_DT_ch = None
 
 
     # not final
     stat_columns = ['ch','rate','realT','liveT','deadT','satT','min_dtus','Nzero','Npu','Ndz','Ntot','Nsat','DT%','DT+S%']
     df_stat = pd.DataFrame( np.nan, index=[0], columns = stat_columns )
     df_stat['ch'] = chan
-    df_stat['Nsat'] = satu_n_ch
-    df_stat['satT'] = satu_DT_ch/1e+6
+    if satu_DT_ch != None:
+        df_stat['Nsat'] = satu_n_ch
+        df_stat['satT'] = satu_DT_ch
+#    else:
+#        df_stat['satT'] = None
+
     df_stat['min_dtus'] = df1.loc[df1['dtus']!=0]['dtus'].min()
     #print( df1.loc[df1['dtus']!=0]['dtus'].min())
     #print( df1['dtus'].min())
@@ -429,18 +437,19 @@ def eva_cut_time(filename,  chan, od=0, do=9999999 ,   tree = False):
     his = general.column_to_histo(  df1["time"].loc[ (df1.E!=0)  ]  , binmax=1+int(df1.iloc[-1]["time"]), himax=1+int(df1.iloc[-1]["time"]), savename = outfile, hname = hname, writeondisk = True, writetxt = False)
 
 
-    #--------------- histogram saturations distribution in time
-    hname = general.generate_hname(filename, chan)+"_saturations"
-    outfile = bname+"saturations_ch"+str(chan)+".txt"
-    #*************
-    his = general.column_to_histo(  df1["time"].loc[ (df1.prev_satu!=0)  ]  , binmax=1+int(df1.iloc[-1]["time"]), himax=1+int(df1.iloc[-1]["time"]), savename = outfile, hname = hname, writeondisk = True, writetxt = False)
+    if 'extras' in  df1:
+        #--------------- histogram saturations distribution in time
+        hname = general.generate_hname(filename, chan)+"_saturations"
+        outfile = bname+"saturations_ch"+str(chan)+".txt"
+        #*************
+        his = general.column_to_histo(  df1["time"].loc[ (df1.prev_satu!=0)  ]  , binmax=1+int(df1.iloc[-1]["time"]), himax=1+int(df1.iloc[-1]["time"]), savename = outfile, hname = hname, writeondisk = True, writetxt = False)
 
 
-    #--------------- histogram saturations length
-    hname = general.generate_hname(filename, chan)+"_satlength"
-    outfile = bname+"satlength_ch"+str(chan)+".txt"
-    #*************
-    his = general.column_to_histo(  df1["prev_satu"].loc[ (df1.prev_satu!=0)  ]  , binmax=100, himax=1+int(df1["prev_satu"].max()), savename = outfile, hname = hname, writeondisk = True, writetxt = False)
+        #--------------- histogram saturations length
+        hname = general.generate_hname(filename, chan)+"_satlength"
+        outfile = bname+"satlength_ch"+str(chan)+".txt"
+        #*************
+        his = general.column_to_histo(  df1["prev_satu"].loc[ (df1.prev_satu!=0)  ]  , binmax=100, himax=1+int(df1["prev_satu"].max()), savename = outfile, hname = hname, writeondisk = True, writetxt = False)
 
 
 
@@ -475,8 +484,12 @@ def eva_cut_time(filename,  chan, od=0, do=9999999 ,   tree = False):
     deadt = dift*deadtpr/100
     livet = dift - deadt
 
-    livetprsat = (dift - deadt - satu_DT_ch/1e+6)
-    deadtprsat = (dift - livetprsat)/dift*100
+    if satu_DT_ch == None:
+        livetprsat = None
+        deadtprsat = None
+    else:
+        livetprsat = (dift - deadt - satu_DT_ch)
+        deadtprsat = (dift - livetprsat)/dift*100
 
 
     stopcut = start + dt.timedelta(seconds=lev)
@@ -497,10 +510,10 @@ def eva_cut_time(filename,  chan, od=0, do=9999999 ,   tree = False):
      zeroes     = {len_dfzero:8d}
      nz events  = {len(df2):8d}
      tot events = {len(df1):8d}
-     saturations= {satu_DT_ch/1e+6:.2f} s
+     saturations= {satu_DT_ch} s
 
      DT %       = {deadtpr:.2f} %
-     DT(inc sat)= {deadtprsat:.2f} %
+     DT(inc sat)= {deadtprsat} %
 """
     print(output)
 
@@ -513,8 +526,13 @@ def eva_cut_time(filename,  chan, od=0, do=9999999 ,   tree = False):
     df_stat['deadT'] = deadt
     df_stat['Nzero'] =len_dfzero
     df_stat['Ntot'] = len(df1)
-    df_stat['iDTavg'] = 100*( (1/rate) * satu_n_ch )/dift   #  average dt * nsatur
-    df_stat['iDTmin'] = 100*( df_stat['min_dtus']*1e-6 * satu_n_ch )/dift  #  mintime * nsatur
+
+    if satu_n_ch != None:
+        df_stat['iDTavg%'] = 100*( (1/rate) * satu_n_ch )/dift   #  average dt * nsatur
+        df_stat['iDTmin%'] = 100*( df_stat['min_dtus']*1e-6 * satu_n_ch )/dift  #  mintime * nsatur
+#    else:
+#        df_stat['iDTavg%'] = None
+#        df_stat['iDTmin%'] = None
     #    df_stat['tSAT'] =
 
 
@@ -551,7 +569,7 @@ def eva_cut_time(filename,  chan, od=0, do=9999999 ,   tree = False):
 
     if tree:
         #*************
-        general.save_to_tree(df2, filename) #
+        general.save_to_tree(df2, filename, treename = f"df{chan}") # no modname...
 
     print( general.freemem() )
 
